// HLK RM04 wi-fi module Common Functions
//
constant RM04 1 // indicater that it is loaded and start of this file
// Hardware
// ES/RST D0
// RX_default RF2  (UART1)
// TX_default RF3
// RX_aux     RF4  (UART2 for pass through)
// TX_aux     RF5
//

//WiFi setup
constant AT_MODE 1
constant DATA_MODE 0

// global
dim eof, atMode

//******************************************************************************
// open port and set up ES/RST pin
// Initialises hardware and clears atMode flag
//******************************************************************************
function init()
dim v
    init_hardware()
    atMode = 0
endf

//******************************************************************************
// For debugging simply gets buffer, the buffer is cleared after this
// operation
//******************************************************************************
function see()
dim k
    while comkey?(CPORT) <> 0
        k = comkey(CPORT)
        if k < 32 then
            print format$("[%d]",k)
        else
            print byte2str(k)
        endif
    wend
    print "\n"
endf

// =============================================================================
// =============================================================================
// String functions
// *****************************************************************************
// returns the position in a UART buffer that the string occures or -1
// port is the uart buffer port
// start is the start position in the buffer, looks foward from there
// str$ is what to find, this can be also "\r" or say "\r\n"
// -- does not consume any of the bufer --
// *****************************************************************************
function bpos(port,start,str$[10])
dim first,j,k, found= -1
dim tmp
    if start < comkey?(port) then // only enough bytes in buffer
        first = asc(str$,start)
        for j = 0 to comkey?(port)
            if compeekp(port,j) = first then
                // check rest of string
                found = j
                if strlen(str$) <> 1 then // found as string is only single
                    for k = 1 to strlen(str$)-1
                        if  compeekp(port,j+k) <> asc(str$,k) then
                            found = -1
                        endif
                    next
                endif
            endif
            // bug in version 2.8-27 cant do break if for within for
            //if found <> -1 then
            //    break
            //endif
        next
    endif
    return found
endf

//******************************************************************************
// gets a line terminated by term, with timeout
// example s$=getLine(10) - where 10 is an LF charcater
//******************************************************************************
function getLine$(term)
dim s$[160]="", k, timeout = 10000
    while timeout > 1
        if comkey?(CPORT) <> 0
            k = comkey(CPORT)
            if k = term
                timeout = 0
            else
                s$ = s$ + byte2str(k)
                timeout = 10000
            endif
        endif
        timeout = timeout - 1
    wend
    return s$
endf

// *****************************************************************************
// trims leading and trailing non printables and space from front and
// back of a string
// *****************************************************************************
function trim$(line$[90])
dim j, rv$[90]=""
    // trim front
    for j = 0 to strlen(line$)
        if asc(line$,j) > 32 then
            break
        endif
    next
    rv$ = mid$(line$,j,200)
    // trim back
    for j = strlen(rv$) to 0 step -1
        //print j, asc(rv$,j)
        if asc(rv$,j) > 32 then
            break
        endif
    next
    rv$ = mid$(rv$,0,j+1)
    return rv$
endf

// =============================================================================
// =============================================================================
// AT_MODE functions
// *****************************************************************************
// sends a command and returns the result
// This adds at+ to the beggining of and \r at the end
// MUST be in mode 1 to work
// Returns
//          result as a string from RM04
// *****************************************************************************
function cmd$(cmd$[90])
dim s$[80] = "at+"+cmd$+"\r"
//print "\nSending",s$
    comclear(CPORT)
    comouts(CPORT,s$)
    getLine$(LF) // get and drop echo
    return trim$(getLine$(LF)) // response
endf

// *****************************************************************************
// sends a command to set a value, uses logging so results are tracked
// Returns 
//          1 on sucess
//          0 on fail
// *****************************************************************************
function set_cmd(value$[90])
dim rv=1
    syslog("set_cmd",format$("Updating %s",value$))
    if trim$(cmd$(value$)) <> "ok" then
        syslog("set_cmd",format$("Possible error when updating %s",value$))
        rv =0
    endif
    return rv
endf

// *****************************************************************************
// checks to see if mode 1 is active, as the module will echo what is sent
// Returns
//          <> 0 if mode 1 is active, 
//          =  0 if it isnt. This can take a few seconds
// *****************************************************************************
function mode1_check()
dim j =4
    while j > 0
        if strlen(cmd$("netmode=?")) > 0 then
            break
        endif
        j = j - 1
        wait(10)
    wend
    if j = 0 then
        atMode =0
    endif
    return j
endf
// *****************************************************************************
// Switches RM04 to mode 1 by asserting the ES pin, then checks for mode1
// and sets the mode 1 flag
// Returns
//          <> 0 on sucess
//          = 0 on fail
// *****************************************************************************
function Mode1()
dim rv
      io_write(ESPORT,ESPIN,LOW)
      wait(100)
      io_write(ESPORT,ESPIN,HIGH)
      wait(100)
      rv = mode1_check()
      if rv > 0 then
        atMode = AT_MODE
      else
        atMode = DATA_MODE
      endif
      return rv
endf

// *****************************************************************************
// Mode 0 is set by sending 'out_trans=0'
// Returns
//          <> 0 on sucess
//          = 0 on fail
// *****************************************************************************
function Mode0()
dim j = 5, s$
    while j > 0
        s$ = cmd$("out_trans=0")
        if s$="ok" then
            atMode = DATA_MODE
            break
        endif
        wait(500)
        j = j - 1
    wend
    return j
endf
    
// *****************************************************************************
// Set data mode, this is used instead of Mode1() and Mode0() as it will
// not attempt to set the mode if already in that mode so can be used
// freely in front of any function that requires a particular mode
// Returns
//          <> 0 on sucess
//          = 0 on fail
// *****************************************************************************
function Mode(m)
dim rv=atMode
    if (atMode <> m) then
        if m = 1 then
            rv = Mode1()
        else
            rv = Mode0()
        endif
    endif
    return rv
endf

// *****************************************************************************
// THIS WILL RESET THE WI-FI MODULE BACK TO ITS DEFAULTS
// It works by holding the ES pin low for a few seconds, the command will 
// take a few seconds to work and also the module will reset, also taking
// about 30 seconds
// *****************************************************************************
function default_reset()
dim j
    io_write(ESPORT,ESPIN,LOW)
    ledCon(0)
    print "\n"
    for j = 1 to 5
        ledL1(1)
        print "."
        wait(2000)
        ledL1(0)
        print "."
        wait(2000)
    next
    io_write(ESPORT,ESPIN,HIGH)
    ledL1(0)
    ledL2(1)
endf

// =============================================================================
// =============================================================================
// Module setting commands
//
// =============================================================================
//
// *****************************************************************************
// reconnect, use when changing any module values, sets mode back to data
// sends 'reconn=1'
// mode will be set to 0 after this command
// *****************************************************************************
function reconnect()
    Mode(AT_MODE)
    if set_cmd("reconn=1") <> 0 then
        atMode=0
    endif
endf

// *****************************************************************************
// sets remote port
// when this is set to AP there will be no internet access
// Returns
//          1 on sucess
//          0 on fail
// *****************************************************************************
function set_remoteport(port)
dim rv=0
    if Mode(AT_MODE) <> 0
        rv = set_cmd("remoteport="+int2str(port))
    endif
    return rv
endf

// *****************************************************************************
// gets current remote port
// Returns
//          port as a string
// *****************************************************************************
function get_remoteport$()
    if Mode(AT_MODE) <> 0
        return cmd$("remoteport=?")
    endif
endf

// *****************************************************************************
// Sets or Gets current remote protocoluse with null string to get value
// values "tcp", "udp"
// Returns
//          current remote protocol
// *****************************************************************************
function set_remotepro$(value$[8])
dim rv$[8]=""
    if Mode(AT_MODE) <> 0
        if strlen(value$) <> 0 then
            if set_cmd("remotepro="+value$) = 1 then
                rv$="ok"
            endif
        else
            rv$ = cmd$("remotepro=?")
        endif
    endif
    return rv$
endf

// *****************************************************************************
// Set or Get remote ip address, use with null string to get value
// example values "192.168.11.54"
// Returns
//          ip address as dotted string
// *****************************************************************************
function set_remoteip$(value$[32])
dim rv$[32]=""
    if Mode(AT_MODE) <> 0
        if strlen(value$) <> 0 then
            if set_cmd("remoteip="+value$) = 1 then
                rv$="ok"
            endif
        else
            rv$ = cmd$("remoteip=?")
        endif
    endif
    return rv$
endf

// *****************************************************************************
// Set or Get newtork mode, use with null string to get value
// when this is set to AP there will be no internet access
// values "none", "client", "server"
// Returns
//          network mode
// *****************************************************************************
function set_mode$(mode$[8])
dim rv$[8]=""
    if Mode(AT_MODE) <> 0
        if strlen(mode$) <> 0 then
            if set_cmd("mode="+mode$) = 1 then
                rv$="ok"
            endif
        else
            rv$ = cmd$("mode=?")
        endif
    endif
    return rv$
endf

// *****************************************************************************
// Set or Get network mode, use with -1 to get value
// when this is set to AP there will be no internet access
// values 0, 1, 2, 3 see RM04 at commands for meaning
// Returns
//          -1 or 0 to 3
// *****************************************************************************
function set_netmode(netmode)
dim rv=-1
    if Mode(AT_MODE) <> 0
        if netmode >= 0 then
            rv = set_cmd("netmode="+netmode)
        else
            rv = cmd$("netmode=?") // will return a number as a string
        endif
    endif
    return rv
endf

// *****************************************************************************
// This is meaningless for AP networking so just return 1
// Returns
//       1 if connected
// *****************************************************************************
function connected()
dim rv = 0
    if Mode(AT_MODE) <> 0
        if set_netmode(-1) <> 3 then
            if cmd$("wifi_ConState=?") = "Connected"
                rv = 1
            endif
        else
            rv = 1
        endif
    endif
    return rv
endf

// =============================================================================
// =============================================================================
// CONFIGURATION
// =============================================================================
// *****************************************************************************
// countdown seconds timer
// blocks until count complete
// *****************************************************************************
function countdown(time)
    print "\n"
    while time <> 0
        print time
        print " "
        wait(1000)
        time = time - 1
    wend
endf

// *****************************************************************************
// gets a line from the configuration file, parses out comments and returns
// with the line. Can return with a blank line so check for this
// eof is a global that is set to <> 0 when end of file is reached
// *****************************************************************************
function nextConfigLine$(f)
dim j, s$[90], line$[90]="", k
    eof = fgets(s$,f)
    if eof = 0 then
        // process line
        line$=""
        for j = 0 to strlen(s$)
            k = asc(s$,j)
            if (k = '#') || (k < 32) then
                break
            else
                line$=line$+chr$(k)
            endif
        next
    endif
    return trim$(line$)
endf

// *****************************************************************************
// splits the config line and returns either the left hand or the right hand
// side of the "="
// *****************************************************************************
function configSplit$(cfg$[90],left)
dim p
    if strlen(cfg$) <> 0 then
        p = pos(cfg$,"=")
            if left = 1 then
                return trim$(mid$(cfg$,0,p))
            else
                return trim$(mid$(cfg$,p+1,100))
            endif
    else
        return ""    
    endif
endf

// *****************************************************************************
// checks config line on wifi module with the one given and returns 1 if
// they are the same
// *****************************************************************************
function checkCfg(cfg$[90])
dim s$[90], rv = 0
    if atMode = 1 then
        if strlen(cfg$) <> 0 then
            s$ = cmd$(configSplit$(cfg$,1) + "=?")
            //print format$("\nConfig Value [%s]",s$)
            if s$ = configSplit$(cfg$,0) then
                rv = 1
            endif
            //print format$(" Split value [%s]",s$)
        endif
        return rv
    else
        syslog("checkCfg","ERROR mode not set")
    endif
endf

// *****************************************************************************
// prints config infromation against a given .cfg file
// example printCfg("default.cfg")
// *****************************************************************************
function printCfg(file$[40])
dim f, s$[90], c$[90], p, commit=0
    if Mode(AT_MODE) <> 0 then
        f = fopen(file$, "r")
        eof = 0
        if f <> 0 then
            while eof = 0
                s$ = nextConfigLine$(f)
                if strlen(s$) <> 0 then
                    c$ = configSplit$(s$,1)
                    print format$("\n%s : ",c$)
                    c$ = cmd$(configSplit$(s$,1) + "=?")
                    print format$("%s",c$)
                endif
            wend
            fclose(f)
        else
            syslog("printCfg", "Error no config file")
        endif
    else
        syslog("printCfg","Cant set mode 1")
    endif
endf

// *****************************************************************************
// updates the module with the given configuration file, it will not update if
// there is nothing to do
// This can be run at start up so that if a cfg file changes the changes
// will be updated
// *****************************************************************************
function updateCfg(file$[40])
dim f, s$[90], p, commit=0
    if Mode(AT_MODE) <> 0 then
        f = fopen(file$, "r")
        eof = 0
        if f <> 0 then
            while eof = 0
                s$ = nextConfigLine$(f)
                if strlen(s$) <> 0 then
                    if checkCfg(s$) <> 1 then
                        syslog("updateCfg",format$("config no match on %s ",s$))
                        syslog("updateCfg","updating")
                        commit = 1
                        set_cmd(s$)
                    endif
                endif
            wend
            fclose(f)
            if commit <> 0 then
                syslog("updateCfg","Commit new settings to wi-fi module")
                cmd$("net_reconn=1")
                cmd$("net_commit=1")
                atMode=0
            endif
        else
            syslog("updateCfg","Error no config file")
        endif
    else
        syslog("updateCfg","Cant set mode 1")
    endif
endf


