// LCD
// This is a very basic display driver.
// modified from http://byvac.com/mBlib/flb/Library/2013/lcd.bas
// Display connection
// 	LCDpin				port:bit	
//	1		GND			GND			
//	2		VCC			+5V	
//	3		Contrast	GND
//	4		RS			RE1			
//  5		RW			RE2
//	6		E			RE0			
//	11		D4			RE3		
//	12		D5			RE4		
//	13		D6			RE5		
//	14		D7			RE6    	
//  15      BL          RE7         
//  16      BL          GND
constant    LCD 1 // indicater that it is loaded and start of this file
//
constant    RS  1
constant    E   0
constant    RW  2
constant    D4  3
constant    D5  4
constant    D6  5
constant    D7  6
constant    BL  7
// *******************************************************************
// Sets ports I/O direction
// *******************************************************************
function lcd_initPorts()
    io_pinMode(PORTE,RS,OUT,WOFF)
    io_pinMode(PORTE,E,OUT,WOFF)
    io_pinMode(PORTE,RW,OUT,WOFF)
    io_pinMode(PORTE,BL,OUT,WOFF)
    io_pinMode(PORTE,D4,OUT,WOFF)
    io_pinMode(PORTE,D5,OUT,WOFF)
    io_pinMode(PORTE,D6,OUT,WOFF)
    io_pinMode(PORTE,D7,OUT,WOFF)
    // important initailiswe with ports low
    io_write(PORTE,RS,LOW)
    io_write(PORTE,E,LOW)
    io_write(PORTE,RW,LOW)
    io_write(PORTE,BL,HIGH) // but turn on BL
    io_write(PORTE,D4,LOW)
    io_write(PORTE,D5,LOW)
    io_write(PORTE,D6,LOW)
    io_write(PORTE,D7,LOW)
endf

// *******************************************************************
// writes nibble to the  4 bits of the lcd port
// *******************************************************************
function lcd_write4(nibble)
	io_write(PORTE,E,HIGH) // set E high
	io_setPortL(PORTE, 0x78) // clear the port bits first
    io_setPortH(PORTE, nibble << 3) // set the 1's		
	io_write(PORTE,E,LOW) // latch it in
endf

// *******************************************************************
// sends data (8 bytes) to the 4 bit lcd port
// *******************************************************************
function lcd_data(d)
	lcd_write4(d >> 4) // high nibble first
	lcd_write4(d)
	io_write(PORTE,RS,HIGH) // default high (cmd sets low)
	wait(10) // no LCD WR line do delay needed - may vary
endf

// *******************************************************************
// lcd command data is sent with RS low
// *******************************************************************
function lcd_cmd(d)
	io_write(PORTE,RS,LOW)
	lcd_data(d)
endf

// *******************************************************************
// initiaalise display
// *******************************************************************
function lcd_init()
	lcd_initPorts()
	wait(20)
	lcd_write4(3)	// still in 8 bit mode
	wait(20)
	lcd_write4(3)	// again
	wait(20)
	lcd_write4(2)	// 4 bit mode
	wait(20)
	io_write(PORTE,RS,HIGH) // default now high
	wait(10)
	lcd_cmd(0x28);
	wait(10);
	lcd_cmd(0x0e);	// display control
	wait(10);
	lcd_cmd(0x06);	// entry mode
	wait(10);
	lcd_cmd(0x01);	// cursor home
	wait(30);
    io_write(PORTE,BL,HIGH)	
endf

// =============================================================================
// Some basic lcd items
// *****************************************************************************
// clear display
// *****************************************************************************
function lcd_cls()
    lcd_cmd(1)
endf

// *****************************************************************************
// send to row, column top row is 0, first column is 0
// *****************************************************************************
function lcd_rc(r,c)
	select(r)
        case(0)
		  c = c + 128
          break
        case(1)
		  c = c + 192
          break
	    case(2)
          c = c + 148
          break
        case(3)
          c = c + 212
          break
        default
          c = c + 128
    endselect
	lcd_cmd(c)	
endf

// *******************************************************************
// sends a string
// *******************************************************************
function lcd_puts(s$[40])
dim j, k=strlen(s$)
	 for j = 0 to k-1
	 	lcd_data(asc(s$,j))
	 next
endf

